@@ -9,7 +9,7 @@ from libnacl.version import __version__
 import ctypes
 import sys
 
-__SONAMES = (13, 10, 5, 4)
+__SONAMES = (17, 13, 10, 5, 4)
 
 
 def _get_nacl():
@@ -29,12 +29,8 @@ def _get_nacl():
                 )
             except OSError:
                 pass
-        try:
-            return ctypes.cdll.LoadLibrary('tweetnacl')
-        except OSError:
-            msg = ('Could not locate nacl lib, searched for libsodium, '
-                   'tweetnacl')
-            raise OSError(msg)
+        msg = 'Could not locate nacl lib, searched for libsodium'
+        raise OSError(msg)
     elif sys.platform.startswith('darwin'):
         try:
             return ctypes.cdll.LoadLibrary('libsodium.dylib')
@@ -46,12 +42,7 @@ def _get_nacl():
                 libpath = __file__[0:libidx+3] + '/libsodium.dylib'
                 return ctypes.cdll.LoadLibrary(libpath)
         except OSError:
-            pass
-        try:
-            return ctypes.cdll.LoadLibrary('tweetnacl.dylib')
-        except OSError:
-            msg = ('Could not locate nacl lib, searched for libsodium, '
-                   'tweetnacl')
+            msg = 'Could not locate nacl lib, searched for libsodium'
             raise OSError(msg)
     else:
         try:
@@ -77,17 +68,25 @@ def _get_nacl():
                 )
             except OSError:
                 pass
-        try:
-            return ctypes.cdll.LoadLibrary('tweetnacl.so')
-        except OSError:
-            msg = 'Could not locate nacl lib, searched for libsodium.so, '
-            for soname_ver in __SONAMES:
-                msg += 'libsodium.so.{0}, '.format(soname_ver)
-            msg += ' and tweetnacl.so'
-            raise OSError(msg)
+        msg = 'Could not locate nacl lib, searched for libsodium.so, '
+        for soname_ver in __SONAMES:
+            msg += 'libsodium.so.{0}, '.format(soname_ver)
+        raise OSError(msg)
 
 nacl = _get_nacl()
 
+
+# Define exceptions
+class CryptError(Exception):
+    """
+    Base Exception for cryptographic errors
+    """
+
+sodium_init = nacl.sodium_init
+sodium_init.res_type = ctypes.c_int
+if sodium_init() < 0:
+    raise RuntimeError('sodium_init() call failed!')
+
 # Define constants
 crypto_box_SECRETKEYBYTES = nacl.crypto_box_secretkeybytes()
 crypto_box_PUBLICKEYBYTES = nacl.crypto_box_publickeybytes()
@@ -123,15 +122,12 @@ crypto_scalarmult_curve25519_BYTES = nacl.crypto_scalarmult_curve25519_bytes()
 crypto_hash_BYTES = nacl.crypto_hash_sha512_bytes()
 crypto_hash_sha256_BYTES = nacl.crypto_hash_sha256_bytes()
 crypto_hash_sha512_BYTES = nacl.crypto_hash_sha512_bytes()
+crypto_verify_16_BYTES = nacl.crypto_verify_16_bytes()
+crypto_verify_32_BYTES = nacl.crypto_verify_32_bytes()
+crypto_verify_64_BYTES = nacl.crypto_verify_64_bytes()
 # pylint: enable=C0103
 
 
-# Define exceptions
-class CryptError(Exception):
-    '''
-    Base Exception for cryptographic errors
-    '''
-
 # Pubkey defs
 
 
@@ -310,23 +306,48 @@ def crypto_sign_open(sig, vk):
 # Authenticated Symmetric Encryption
 
 
-def crypto_secretbox(msg, nonce, key):
-    '''
-    Encrypts and authenticates a message using the given secret key, and nonce
-    '''
-    pad = b'\x00' * crypto_secretbox_ZEROBYTES + msg
+def crypto_secretbox(message, nonce, key):
+    """Encrypts and authenticates a message using the given secret key, and nonce
+
+    Args:
+        message (bytes): a message to encrypt
+        nonce (bytes): nonce, does not have to be confidential must be
+            `crypto_secretbox_NONCEBYTES` in length
+        key (bytes): secret key, must be `crypto_secretbox_KEYBYTES` in
+            length
+
+    Returns:
+        bytes: the ciphertext
+
+    Raises:
+        ValueError: if arguments' length is wrong or the operation has failed.
+    """
+    if len(key) != crypto_secretbox_KEYBYTES:
+        raise ValueError('Invalid key')
+
+    if len(nonce) != crypto_secretbox_NONCEBYTES:
+        raise ValueError('Invalid nonce')
+
+    pad = b'\x00' * crypto_secretbox_ZEROBYTES + message
     ctxt = ctypes.create_string_buffer(len(pad))
-    ret = nacl.crypto_secretbox(ctxt, pad, ctypes.c_ulonglong(len(pad)), nonce, key)
+    ret = nacl.crypto_secretbox(
+        ctxt, pad, ctypes.c_ulonglong(len(pad)), nonce, key)
     if ret:
         raise ValueError('Failed to encrypt message')
     return ctxt.raw[crypto_secretbox_BOXZEROBYTES:]
 
 
 def crypto_secretbox_open(ctxt, nonce, key):
-    '''
+    """
     Decrypts a ciphertext ctxt given the receivers private key, and senders
     public key
-    '''
+    """
+    if len(key) != crypto_secretbox_KEYBYTES:
+        raise ValueError('Invalid key')
+
+    if len(nonce) != crypto_secretbox_NONCEBYTES:
+        raise ValueError('Invalid nonce')
+
     pad = b'\x00' * crypto_secretbox_BOXZEROBYTES + ctxt
     msg = ctypes.create_string_buffer(len(pad))
     ret = nacl.crypto_secretbox_open(
@@ -401,27 +422,71 @@ def crypto_auth_verify(tok, msg, key):
 # One time authentication
 
 
-def crypto_onetimeauth(msg, key):
-    '''
-    Constructs a one time authentication token for the given message msg using
+def crypto_onetimeauth_primitive():
+    """
+    Return the onetimeauth underlying primitive
+
+    Returns:
+        str: always ``poly1305``
+    """
+    func = nacl.crypto_onetimeauth_primitive
+    func.restype = ctypes.c_char_p
+    return func().decode()
+
+
+def crypto_onetimeauth(message, key):
+    """
+    Constructs a one time authentication token for the given message using
     a given secret key
-    '''
+
+    Args:
+        message (bytes): message to authenticate.
+        key (bytes): secret key - must be of crypto_onetimeauth_KEYBYTES length.
+
+    Returns:
+        bytes: an authenticator, of crypto_onetimeauth_BYTES length.
+
+    Raises:
+        ValueError: if arguments' length is wrong.
+    """
+    if len(key) != crypto_onetimeauth_KEYBYTES:
+        raise ValueError('Invalid secret key')
+
     tok = ctypes.create_string_buffer(crypto_onetimeauth_BYTES)
-    ret = nacl.crypto_onetimeauth(tok, msg, ctypes.c_ulonglong(len(msg)), key)
-    if ret:
-        raise ValueError('Failed to auth msg')
+    # cannot fail
+    _ = nacl.crypto_onetimeauth(
+        tok, message, ctypes.c_ulonglong(len(message)), key)
+
     return tok.raw[:crypto_onetimeauth_BYTES]
 
 
-def crypto_onetimeauth_verify(tok, msg, key):
-    '''
-    Verifies that the given authentication token is correct for the given
-    message and key
-    '''
-    ret = nacl.crypto_onetimeauth_verify(tok, msg, ctypes.c_ulonglong(len(msg)), key)
+def crypto_onetimeauth_verify(token, message, key):
+    """
+    Verifies, in constant time, that ``token`` is a correct authenticator for
+    the message using the secret key.
+
+    Args:
+        token (bytes): an authenticator of crypto_onetimeauth_BYTES length.
+        message (bytes): The message to authenticate.
+        key: key (bytes): secret key - must be of crypto_onetimeauth_KEYBYTES
+            length.
+
+    Returns:
+        bytes: secret key - must be of crypto_onetimeauth_KEYBYTES length.
+
+    Raises:
+        ValueError: if arguments' length is wrong or verification has failed.
+    """
+    if len(key) != crypto_onetimeauth_KEYBYTES:
+        raise ValueError('Invalid secret key')
+    if len(token) != crypto_onetimeauth_BYTES:
+        raise ValueError('Invalid authenticator')
+
+    ret = nacl.crypto_onetimeauth_verify(
+        token, message, ctypes.c_ulonglong(len(message)), key)
     if ret:
-        raise ValueError('Failed to auth msg')
-    return msg
+        raise ValueError('Failed to auth message')
+    return message
 
 # Hashing
 
@@ -516,8 +581,21 @@ def crypto_verify_32(string1, string2):
     return not nacl.crypto_verify_32(string1, string2)
 
 
+def crypto_verify_64(string1, string2):
+    '''
+    Compares the first crypto_verify_64_BYTES of the given strings
+
+    The time taken by the function is independent of the contents of string1
+    and string2. In contrast, the standard C comparison function
+    memcmp(string1,string2,64) takes time that is dependent on the longest
+    matching prefix of string1 and string2. This often allows for easy
+    timing attacks.
+    '''
+    return not nacl.crypto_verify_64(string1, string2)
+
 # Random byte generation
 
+
 def randombytes(size):
     '''
     Return a string of random bytes of the given size
